<?php

namespace Drupal\wse_lb\EventSubscriber;

use Drupal\block_content\BlockContentInterface;
use Drupal\Core\Access\AccessibleInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\layout_builder\EventSubscriber\SetInlineBlockDependency as LBSetInlineBlockDependency;

/**
 * Set block dependency by entity ID instead of revision ID.
 *
 * WSE LB works against entity IDs instead of revisions, this class keeps inline
 * block dependency setting working similarly to how the core solution works
 * except that we leverage IDs instead of revisions to do the loading process.
 *
 * The docs on Drupal\layout_builder\EventSubscriber\SetInlineBlockDependency
 * have more details as to why this is necessary.
 *
 * @internal
 *   Tagged services are internal.
 *
 * @see \Drupal\layout_builder\EventSubscriber\SetInlineBlockDependency
 * @see \Drupal\file\FileAccessControlHandler::checkAccess()
 * @see \Drupal\block_content\BlockContentAccessControlHandler::checkAccess()
 */
class SetInlineBlockDependency extends LBSetInlineBlockDependency {

  /**
   * Get the access dependency of an inline block.
   *
   * If the block is used in an entity that entity will be returned as the
   * dependency.
   *
   * For revisionable entities the entity will only be returned if it is used in
   * the latest revision of the entity. For inline blocks that are not used in
   * the latest revision but are used in a previous revision the entity will not
   * be returned because calling
   * \Drupal\Core\Access\AccessibleInterface::access() will only check access on
   * the latest revision. Therefore if the previous revision of the entity was
   * returned as the dependency access would be granted to inline block
   * regardless of whether the user has access to the revision in which the
   * inline block was used.
   *
   * @param \Drupal\block_content\BlockContentInterface $block_content
   *   The block content entity.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   Returns the layout dependency.
   *
   * @see \Drupal\block_content\BlockContentAccessControlHandler::checkAccess()
   * @see \Drupal\layout_builder\EventSubscriber\BlockComponentRenderArray::onBuildRender()
   */
  protected function getInlineBlockDependency(BlockContentInterface $block_content, string $operation = NULL): ?AccessibleInterface {
    $layout_entity_info = $this->usage->getUsage($block_content->id());
    if (empty($layout_entity_info)) {
      // If the block does not have usage information then we cannot set a
      // dependency. It may be used by another module besides Layout Builder.
      return NULL;
    }
    $layout_entity_storage = \Drupal::entityTypeManager()->getStorage($layout_entity_info->layout_entity_type);
    $layout_entity = $layout_entity_storage->load($layout_entity_info->layout_entity_id);
    if ($this->isLayoutCompatibleEntity($layout_entity)) {
      if ($this->isBlockIdUsedInEntity($layout_entity, $block_content)) {
        return $layout_entity;
      }
    }

    return NULL;
  }

  /**
   * Determines if a block content id is used in an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $layout_entity
   *   The layout entity.
   * @param \Drupal\block_content\BlockContentInterface $block_content
   *   The block content entity.
   *
   * @return bool
   *   TRUE if the block content entity is used as an inline block in the
   *   layout entity.
   */
  protected function isBlockIdUsedInEntity(EntityInterface $layout_entity, BlockContentInterface $block_content) {
    $sections_blocks_ids = $this->getInlineBlockIdsInSections($this->getEntitySections($layout_entity));
    return in_array($block_content->id(), $sections_blocks_ids, TRUE);
  }

  /**
   * Gets block IDs for layout sections.
   *
   * @param \Drupal\layout_builder\Section[] $sections
   *   The layout sections.
   *
   * @return int[]
   *   The block IDs.
   */
  protected function getInlineBlockIdsInSections(array $sections) {
    $ids = [];
    foreach ($this->getInlineBlockComponents($sections) as $component) {
      $configuration = $component->getPlugin()->getConfiguration();
      if (!empty($configuration['block_id'])) {
        $ids[] = $configuration['block_id'];
      }
    }
    return $ids;
  }

}
