<?php

namespace Drupal\wse_lb\EventSubscriber;

use Drupal\Core\Form\FormStateInterface;
use Drupal\layout_builder\SectionStorageInterface;
use Drupal\workspaces\Event\WorkspaceSafeFormEvent;
use Drupal\workspaces\WorkspaceInformationInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Provides an event subscriber for setting workspace-safe forms.
 */
class WorkspaceSafeForms implements EventSubscriberInterface {

  /**
   * An array of Layout Builder form IDs.
   *
   * @var array
   */
  const FORM_IDS = [
    'layout_builder_add_block',
    'layout_builder_block_move',
    'layout_builder_update_block',
    'layout_builder_configure_block',
    'layout_builder_configure_section',
    'layout_builder_disable_form',
    'layout_builder_remove_block',
    'layout_builder_remove_section',
    'layout_builder_revert_overrides',
    'layout_builder_discard_changes',
  ];

  public function __construct(
    protected readonly WorkspaceInformationInterface $workspaceInfo
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[WorkspaceSafeFormEvent::class][] = 'onWorkspaceSafeForm';
    return $events;
  }

  /**
   * Marks the safeness of layout_builder forms when they are safe.
   *
   * @param \Drupal\workspaces\Event\WorkspaceSafeFormEvent $event
   *   The workspace safe form event.
   */
  public function onWorkspaceSafeForm(WorkspaceSafeFormEvent $event): void {
    if (!in_array($event->getFormId(), static::FORM_IDS, TRUE)) {
      return;
    }

    if ($section_storage = $this->getSectionStorage($event->getFormState())) {
      $context_definitions = $section_storage->getContextDefinitions();
      if (!empty($context_definitions['entity'])) {
        /** @var \Drupal\Core\Entity\EntityInterface $entity */
        $entity = $section_storage->getContext('entity')->getContextValue();
        if ($entity && $this->workspaceInfo->isEntityTypeSupported($entity->getEntityType())) {
          $event->setSafe(TRUE);
          $event->stopPropagation();
        }
      }
    }
  }

  /**
   * Retrieves the section storage from a form state object, if it exists.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return \Drupal\layout_builder\SectionStorageInterface|null
   *   The section storage or NULL if it doesn't exist.
   */
  protected function getSectionStorage(FormStateInterface $form_state) {
    foreach ($form_state->getBuildInfo()['args'] as $argument) {
      if ($argument instanceof SectionStorageInterface) {
        return $argument;
      }
    }
  }

}
