<?php

namespace Drupal\wse_menu\EventSubscriber;

use Drupal\Core\Menu\MenuLinkManagerInterface;
use Drupal\Core\Menu\StaticMenuLinkOverridesInterface;
use Drupal\Core\State\StateInterface;
use Drupal\views\Entity\View;
use Drupal\workspaces\Event\WorkspacePostPublishEvent;
use Drupal\workspaces\Event\WorkspacePrePublishEvent;
use Drupal\workspaces\WorkspaceManagerInterface;
use Drupal\wse_menu\Entity\WseMenuTree;
use Drupal\wse_menu\WseViewsMenuLink;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Wse config event subscriber.
 */
class WseMenuSubscriber implements EventSubscriberInterface {

  /**
   * Constructs event subscriber.
   */
  public function __construct(
    protected WorkspaceManagerInterface $workspaceManager,
    protected StaticMenuLinkOverridesInterface $staticMenuLinkOverrides,
    protected MenuLinkManagerInterface $menuLinkManager,
    protected StateInterface $state
  ) {}

  /**
   * Publishes workspace-specific menu link overrides.
   *
   * @param \Drupal\workspaces\Event\WorkspacePrePublishEvent $event
   *    The workspace pre-publish event.
   */
  public function onWorkspacePrePublish(WorkspacePrePublishEvent $event): void {
    $overrides = $this->workspaceManager->executeInWorkspace($event->getWorkspace()->id(), function () {
      /** @var \Drupal\Core\Menu\StaticMenuLinkOverridesInterface $static_menu_link_overrides */
      $static_menu_link_overrides = \Drupal::service('menu_link.static.overrides');

      $definitions = WseMenuTree::load(1)->getDefinitions();
      return $static_menu_link_overrides->loadMultipleOverrides(array_keys($definitions));
    });


    $this->workspaceManager->executeOutsideWorkspace(function () use ($overrides) {
      /** @var \Drupal\Core\Menu\MenuLinkManagerInterface $menu_link_manager */
      $menu_link_manager = \Drupal::service('plugin.manager.menu.link');

      foreach ($overrides as $id => $override) {
        $menu_link_manager->updateDefinition($id, $override);
      }
    });
  }

  /**
   * Publishes workspace-specific menu link overrides.
   *
   * @param \Drupal\workspaces\Event\WorkspacePostPublishEvent $event
   *    The workspace post-publish event.
   */
  public function onWorkspacePostPublish(WorkspacePostPublishEvent $event): void {
    $workspace_id = $event->getWorkspace()->id();
    $all_overrides = $this->state->get(WseViewsMenuLink::STATE_KEY, []);

    if (isset($all_overrides[$workspace_id])) {
      $overrides = $all_overrides[$workspace_id];
      $this->workspaceManager->executeOutsideWorkspace(function () use ($overrides) {
        foreach ($overrides as $view_id => $displays) {
          if (!$view_entity = View::load($view_id)) {
            continue;
          }

          $view = $view_entity->getExecutable();
          foreach ($displays as $display_id => $display_overrides) {
            $view->setDisplay($display_id);
            $view->initDisplay();

            $display = &$view->storage->getDisplay($view->current_display);
            foreach ($display_overrides as $key => $new_definition_value) {
              $display['display_options']['menu'][$key] = $new_definition_value;
            }
          }

          $view->storage->save();
        }
      });

      // Remove the override info for the published workspace.
      unset($all_overrides[$workspace_id]);
      $this->state->set(WseViewsMenuLink::STATE_KEY, $all_overrides);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[WorkspacePrePublishEvent::class][] = 'onWorkspacePrePublish';
    $events[WorkspacePostPublishEvent::class][] = 'onWorkspacePostPublish';
    return $events;
  }

}
