<?php

namespace Drupal\Tests\wse_menu\Kernel;

use Drupal\KernelTests\Core\Menu\MenuTreeStorageTest;
use Drupal\Tests\user\Traits\UserCreationTrait;
use Drupal\Tests\workspaces\Kernel\WorkspaceTestTrait;
use Drupal\workspaces\Entity\Workspace;
use Drupal\wse_menu\WseMenuTreeStorage;

/**
 * Tests the WSE menu tree storage.
 *
 * @group wse_menu
 */
class WseMenuTreeStorageTest extends MenuTreeStorageTest {

  use UserCreationTrait;
  use WorkspaceTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'link',
    'system',
    'user',
    'workspaces',
    'wse',
    'wse_menu',
    'wse_menu_test',
    'path_alias',
    'options',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->entityTypeManager = \Drupal::entityTypeManager();
    $this->workspaceManager = \Drupal::service('workspaces.manager');

    $this->installSchema('system', ['sequences', 'sessions']);
    $this->installSchema('user', ['users_data']);
    $this->installSchema('workspaces', ['workspace_association']);
    $this->installEntitySchema('user');
    $this->installEntitySchema('workspace');
    $this->installEntitySchema('wse_menu_tree');

    $this->workspaces['stage'] = Workspace::create(['id' => 'stage', 'label' => 'Stage']);
    $this->workspaces['stage']->save();

    $permissions = array_intersect([
      'administer nodes',
      'create workspace',
      'edit any workspace',
      'view any workspace',
    ], array_keys($this->container->get('user.permissions')->getPermissions()));
    $this->setCurrentUser($this->createUser($permissions));

    // Swap the tree storage class instantiated by the parent with our storage.
    $this->treeStorage = new WseMenuTreeStorage(
      $this->treeStorage,
      $this->container->get('workspaces.manager'),
      $this->container->get('entity_type.manager')
    );

    // Activate the Stage workspace, so all test methods of the parent class run
    // in a workspace context.
    $this->switchToWorkspace($this->workspaces['stage']->id());
  }

  /**
   * {@inheritdoc}
   */
  public function testLoadByProperties() {
    parent::testLoadByProperties();

    // Test loading by route parameters.
    $this->addMenuLink('test_link.2', '', 'test', ['foo' => 'bar'], 'menu1');
    $properties = ['route_parameters' => ['foo' => 'bar']];
    $links = $this->treeStorage->loadByProperties($properties);
    $this->assertEquals('menu1', $links['test_link.2']['menu_name']);
    $this->assertEquals('test', $links['test_link.2']['route_name']);
  }

  /**
   * {@inheritdoc}
   */
  protected function addMenuLink($id, $parent = '', $route_name = 'test', $route_parameters = [], $menu_name = 'tools', $extra = []) {
    // Add some defaults which are usually provided by the menu link manager.
    $extra += [
      'enabled' => 1,
      'weight' => 0,
    ];

    parent::addMenuLink($id, $parent, $route_name, $route_parameters, $menu_name, $extra);
  }

  /**
   * {@inheritdoc}
   */
  protected function assertMenuLink(string $id, array $expected_properties, array $parents = [], array $children = []): void {
    $all = $this->treeStorage->loadByProperties(['id' => $id] + $expected_properties, TRUE);
    $this->assertCount(1, $all, "Found link $id matching all the expected properties");
    $raw = reset($all);

    // Put the current link onto the front.
    array_unshift($parents, $raw['id']);

    $found_parents = $this->treeStorage->loadMultiple($parents);
    $this->assertSameSize($parents, $found_parents, 'Found expected number of parents');
    $this->assertCount($raw['depth'], $found_parents, 'Number of parents is the same as the depth');

    $materialized_path = $this->treeStorage->getRootPathIds($id);
    $this->assertEquals(array_values($parents), array_values($materialized_path), 'Parents match the materialized path');

    for ($i = $raw['depth']; $i >= 1; $i--) {
      array_shift($parents);
    }

    if ($parents) {
      $this->assertEquals(end($parents), $raw['parent'], 'Ensure that the parent field is set properly');
    }
    // Verify that the child IDs match.
    $this->assertEqualsCanonicalizing($children, array_keys($this->treeStorage->loadAllChildren($id)));
  }

}
