<?php

namespace Drupal\wse_preview\Negotiator;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\KeyValueStore\KeyValueExpirableFactoryInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Session\AccountInterface;
use Drupal\workspaces\Negotiator\WorkspaceNegotiatorInterface;
use Drupal\workspaces\WorkspaceInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Defines the session workspace negotiator.
 */
class CookieWorkspaceNegotiator implements WorkspaceNegotiatorInterface, EventSubscriberInterface {

  /**
   * Constructor.
   */
  public function __construct(
    protected readonly AccountInterface $currentUser,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly RequestStack $requestStack,
    protected readonly KeyValueExpirableFactoryInterface $keyValueExpirableFactory,
    protected readonly KillSwitch $pageCacheKillSwitch
  ) { }

  /**
   * {@inheritdoc}
   */
  public function applies(Request $request) {
    // This negotiator only applies if a 'wse_preview' cookie exists.
    return $request->cookies->has('wse_preview');
  }

  /**
   * {@inheritdoc}
   */
  public function getActiveWorkspace(Request $request) {
    $preview_id = $request->cookies->get('wse_preview');
    $preview = $this->keyValueExpirableFactory->get('wse_preview')->get($preview_id);

    if (!$preview) {
      return NULL;
    }

    $workspace = $this->entityTypeManager->getStorage('workspace')->load($preview['workspace']);
    if ($workspace instanceof WorkspaceInterface && wse_workspace_get_status($workspace) != WSE_STATUS_CLOSED) {
      // Ensure that pages accessed through workspace previews are not cached.
      $this->pageCacheKillSwitch->trigger();

      return $workspace;
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function setActiveWorkspace(WorkspaceInterface $workspace) {
    // This negotiator can not set the active workspace by itself.
  }

  /**
   * {@inheritdoc}
   */
  public function unsetActiveWorkspace() {
    $this->requestStack->getMainRequest()->getSession()->set('clear_wse_preview', TRUE);
  }

  /**
   * Removes the 'wse_preview' cookie if needed.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The event to process.
   */
  public function onResponse(ResponseEvent $event) {
    if (!$event->isMainRequest()) {
      return;
    }
    if ($event->getRequest()->getSession()->has('clear_wse_preview')) {
      $event->getResponse()->headers->clearCookie('wse_preview');
      $event->getResponse()->headers->clearCookie('NO_CACHE');
      $event->getRequest()->getSession()->remove('clear_wse_preview');
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    $events[KernelEvents::RESPONSE][] = ['onResponse'];
    return $events;
  }

}
