<?php

namespace Drupal\wse\Controller;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\node\Controller\NodeController;
use Drupal\node\NodeStorageInterface;
use Drupal\node\NodeInterface;
use Drupal\workspaces\Plugin\Validation\Constraint\EntityWorkspaceConflictConstraint;
use Drupal\workspaces\WorkspaceManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Overrides core's NodeController.
 */
class WseNodeController extends NodeController implements ContainerInjectionInterface {

  /**
   * The workspace manager.
   *
   * @var \Drupal\workspaces\WorkspaceManagerInterface
   */
  protected WorkspaceManagerInterface $workspaceManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->workspaceManager = $container->get('workspaces.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function revisionOverview(NodeInterface $node) {
    $build = parent::revisionOverview($node);

    if ($this->workspaceManager->hasActiveWorkspace()) {
      $build['node_revisions_table']['#empty'] = $this->t('There are no revisions yet in this workspace.');
      $build['#cache']['contexts'][] = 'workspace';
    }

    // Run the workspace conflict validation constraint when the entity form is
    // being built, so we can "disable" it early and display a message to the
    // user, instead of allowing them to enter data that can never be saved.
    foreach ($node->getTypedData()->getConstraints() as $constraint) {
      if ($constraint instanceof EntityWorkspaceConflictConstraint) {
        $violations = \Drupal::typedDataManager()->getValidator()->validate(
          $node->getTypedData(),
          $constraint
        );
        if ($violations->count()) {
          $build['node_revisions_table']['#access'] = FALSE;
          $build['workspace_conflict']['#markup'] = $violations->get(0)->getMessage();
        }
      }
    }

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function getRevisionIds(NodeInterface $node, NodeStorageInterface $node_storage) {
    $query = $node_storage->getQuery()
      ->accessCheck(TRUE)
      ->allRevisions()
      ->condition($node->getEntityType()->getKey('id'), $node->id())
      ->sort($node->getEntityType()->getKey('revision'), 'DESC')
      ->pager(50);

    // When a workspace is active, filter the revisions to those created in that
    // workspace.
    if ($this->workspaceManager->hasActiveWorkspace()) {
      $active_workspace_id = $this->workspaceManager->getActiveWorkspace()->id();
      $query->condition($node->getEntityType()->getRevisionMetadataKey('workspace'), $active_workspace_id);
    }
    else {
      // Otherwise show only revisions that were not created in a workspace.
      $query->notExists($node->getEntityType()->getRevisionMetadataKey('workspace'));
    }

    $result = $query->execute();
    return array_keys($result);
  }

}
