<?php

namespace Drupal\wse\Core\Event;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\EventDispatcher\Event;

/**
 * An event used to determine whether a form is safe to submit in a workspace.
 */
class WorkspaceSafeFormEvent extends Event {

  /**
   * The form definition.
   *
   * @var array
   */
  protected array $form;

  /**
   * The form state object.
   *
   * @var \Drupal\Core\Form\FormStateInterface
   */
  protected FormStateInterface $formState;

  /**
   * The form ID.
   *
   * @var string
   */
  protected string $formId;

  /**
   * Whether or not the form can be submitted in a workspace.
   *
   * @var bool
   */
  protected bool $isSafe = FALSE;

  /**
   * WorkspaceSafeFormEvent constructor.
   *
   * @param array $form
   *   The form definition.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param string $form_id
   *   The form ID.
   */
  public function __construct(array $form, FormStateInterface $form_state, string $form_id) {
    $this->form = $form;
    $this->formState = $form_state;
    $this->formId = $form_id;
  }

  /**
   * Gets the form array.
   *
   * @return array
   */
  public function getForm(): array {
    return $this->form;
  }

  /**
   * Gets the form state object.
   *
   * @return \Drupal\Core\Form\FormStateInterface
   */
  public function getFormState(): FormStateInterface {
    return $this->formState;
  }

  /**
   * Gets the form ID.
   *
   * @return string
   */
  public function getFormId(): string {
    return $this->formId;
  }

  /**
   * Sets whether or not a form is workspace-safe.
   *
   * @param bool $safe
   *   The workspace-safe value for a form.
   */
  public function setSafe(bool $safe): void {
    $this->isSafe = $safe;
  }

  /**
   * Gets the workspace-safe value for a form.
   *
   * @return bool
   */
  public function isSafe(): bool {
    return $this->isSafe;
  }

}
