<?php

namespace Drupal\wse\Core;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\workspaces\WorkspaceAssociationInterface;
use Drupal\workspaces\WorkspaceInterface;

/**
 * General service for workspace support information.
 */
class WorkspaceInformation implements WorkspaceInformationInterface {

  /**
   * An array of which entity types are supported.
   *
   * @var string[]
   */
  protected array $supported = [];

  /**
   * An array of which entity types are ignored.
   *
   * @var string[]
   */
  protected array $ignored = [];

  public function __construct(
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly WorkspaceAssociationInterface $workspaceAssociation
  ) {}

  /**
   * {@inheritdoc}
   */
  public function isEntitySupported(EntityInterface $entity): bool {
    $entity_type = $entity->getEntityType();

    if ($entity_type->get('workspace') === static::SUPPORTED_CUSTOM) {
      /** @var \Drupal\workspaces\Entity\Handler\WorkspaceHandlerInterface $handler */
      $handler = $this->entityTypeManager->getHandler($entity_type->id(), 'workspace');

      return $handler->isEntitySupported($entity);
    }

    return $this->isEntityTypeSupported($entity_type);
  }

  /**
   * {@inheritdoc}
   */
  public function isEntityTypeSupported(EntityTypeInterface $entity_type): bool {
    $entity_type_id = $entity_type->id();
    if (!isset($this->supported[$entity_type_id])) {
      if ($workspace_support = $entity_type->get('workspace')) {
        $this->supported[$entity_type_id] = $workspace_support === static::SUPPORTED || $workspace_support === static::SUPPORTED_CUSTOM;
      }
      else {
        // Add a fallback check for entity types which might not have gone through
        // the entity type manager's building phase.
        $this->supported[$entity_type_id] = $entity_type->entityClassImplements(EntityPublishedInterface::class) && $entity_type->isRevisionable();
      }
    }
    return $this->supported[$entity_type_id];
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedEntityTypes(): array {
    $entity_types = [];
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      if ($this->isEntityTypeSupported($entity_type)) {
        $entity_types[$entity_type_id] = $entity_type;
      }
    }
    return $entity_types;
  }

  /**
   * {@inheritdoc}
   */
  public function isEntityIgnored(EntityInterface $entity): bool {
    $entity_type = $entity->getEntityType();

    if ($entity_type->get('workspace') === static::SUPPORTED_CUSTOM) {
      /** @var \Drupal\workspaces\Entity\Handler\WorkspaceHandlerInterface $handler */
      $handler = $this->entityTypeManager->getHandler($entity_type->id(), 'workspace');

      return $handler->isEntitySupported($entity) === FALSE;
    }

    return $this->isEntityTypeIgnored($entity_type);
  }

  /**
   * {@inheritdoc}
   */
  public function isEntityTypeIgnored(EntityTypeInterface $entity_type): bool {
    $entity_type_id = $entity_type->id();
    if (!isset($this->ignored[$entity_type_id])) {
      $this->ignored[$entity_type_id] = $entity_type->get('workspace') === static::IGNORED;
    }
    return $this->ignored[$entity_type_id];
  }

  /**
   * {@inheritdoc}
   */
  public function isEntityDeletable(EntityInterface $entity, WorkspaceInterface $workspace): bool {
    // @todo Move this to a service decorator.
    // Workspace config is a special case because it's never relevant in the
    // Live workspace.
    if ($entity->getEntityTypeId() == 'wse_config') {
      return TRUE;
    }

    $initial_revisions = $this->workspaceAssociation->getAssociatedInitialRevisions($workspace->id(), $entity->getEntityTypeId());

    return in_array($entity->id(), $initial_revisions, TRUE);
  }

}
