<?php

namespace Drupal\wse\Core;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\workspaces\WorkspaceInterface;

/**
 * Provides an interface for workspace support information.
 */
interface WorkspaceInformationInterface {

  /**
   * Indicates that CRUD operations for a non-supported entity type are allowed.
   */
  const IGNORED = 'ignored';

  /**
   * Indicates that an entity type can belong to a workspace.
   */
  const SUPPORTED = 'supported';

  /**
   * Indicates that some entities of a certain type may belong to a workspace.
   *
   * Entity types that need to check workspace support on a per-entity basis
   * can do so by implementing a 'workspace' entity type handler.
   *
   * @see \Drupal\workspaces\Entity\Handler\BlockContentWorkspaceHandler
   */
  const SUPPORTED_CUSTOM = 'supported_custom';

  /**
   * Determines if an individual entity can belong to a workspace.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to check.
   *
   * @return bool
   *   TRUE if the entity can belong to a workspace, FALSE otherwise.
   */
  public function isEntitySupported(EntityInterface $entity): bool;

  /**
   * Determines if an entity type can belong to a workspace.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type to check.
   *
   * @return bool
   *   TRUE if the entity type can belong to a workspace, FALSE otherwise.
   */
  public function isEntityTypeSupported(EntityTypeInterface $entity_type): bool;

  /**
   * Returns an array of entity types that can belong to workspaces.
   *
   * @return \Drupal\Core\Entity\EntityTypeInterface[]
   *   The entity types that can belong to workspaces.
   */
  public function getSupportedEntityTypes(): array;

  /**
   * Determines if CRUD operations for an entity are allowed.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to check.
   *
   * @return bool
   *   TRUE if CRUD operations of an entity type can safely be done inside a
   *   workspace, without impacting the Live site, FALSE otherwise.
   */
  public function isEntityIgnored(EntityInterface $entity): bool;

  /**
   * Determines if CRUD operations for a non-supported entity type are allowed.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type to check.
   *
   * @return bool
   *   TRUE if CRUD operations of an entity type can safely be done inside a
   *   workspace, without impacting the Live site, FALSE otherwise.
   */
  public function isEntityTypeIgnored(EntityTypeInterface $entity_type): bool;

  /**
   * Determines whether an entity can be deleted in the given workspace.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object which needs to be checked.
   * @param \Drupal\workspaces\WorkspaceInterface $workspace
   *   The workspace in which the entity needs to be checked.
   *
   * @return bool
   *   TRUE if the entity can be deleted, FALSE otherwise.
   */
  public function isEntityDeletable(EntityInterface $entity, WorkspaceInterface $workspace): bool;

}
